SuperStrict

Import "bluemarble.o"
Import sidesign.minib3d
Import muttley.INIFileHandler

AppTitle = "Krischans Blue Marble"

Local configFile:TINIFile = TINIFile.Create("bluemarble.ini")

Global ScreenX:Int = 1024							' Screen Resolution X
Global ScreenY:Int = 768							' Screen Resolution Y
Global ScreenMode:Int = 1							' Screen Mode
Global Music:String = "Zeropage - Ambient Flight (Electro Vocal Mix).ogg"	' Background Music File

Global fader:Float = 1.0

Const PR:Int = 128 ' planet colors RED
Const PG:Int = 192 ' planet colors GREEN
Const PB:Int = 255 ' planet colors BLUE

If Not configFile.Load() Then

	Local missingConfigFile:TINIFile = TINIFile.Create("bluemarble.ini")
	missingConfigFile.Load()
	missingConfigFile.CreateMissingEntries(True)
	missingConfigFile.SetIntValue("Screen", "Width", ScreenX)
	missingConfigFile.SetIntValue("Screen", "Height", ScreenY)
	missingConfigFile.SetIntValue("Screen", "Mode", ScreenMode)
	missingConfigFile.SetStringValue("Music", "Music", Music)
	missingConfigFile.Save(False)

Else

	configFile.CreateMissingEntries(True)
	ScreenX = configFile.GetIntValue("Screen", "Width", ScreenX)
	ScreenY = configFile.GetIntValue("Screen", "Height", ScreenY)
	ScreenMode = configFile.GetIntValue("Screen", "Mode", ScreenMode)
	Music = configFile.GetStringValue("Music", "Music", Music)
	configFile.Save(False)

End If

' External Win32 Functions to set Window Icon
Extern "win32"
	Function ExtractIconA:Int(hWnd:Int, File$z, Index:Int)
	Function GetActiveWindow%()
	Function SendMessage:Int(hWnd:Int, MSG:Int, wParam:Int, lParam:Int) = "SendMessageA@16"
End Extern

' MiniB3D start		
Graphics3D ScreenX, ScreenY, 32, ScreenMode

Include "include/milkyway.bmx"

' set Window Icon
SetIcon(AppFile, GetActiveWindow())

ClearTextureFilters()

' media
Global earthtex:TTexture = LoadTexture("textures/earth.jpg", 8)
Global nighttex:TTexture = LoadTexture("textures/earthnight.jpg", 8)
Global cloudtex:TTexture = LoadTexture("textures/clouds.png", 2 + 8)
Global moontex:TTexture = LoadTexture("textures/moon.jpg", 8)
Global atmotex:TTexture = LoadTexture("textures/atmo.png", 8)
Global masktex:TTexture = LoadTexture("textures/earthmask.png", 8)

Global backgroundmusic:TSound = LoadSound("music/" + Music, True)

' pivots
Global cpiv:TPivot = CreatePivot()
Global mpiv:TPivot = CreatePivot(cpiv)

' camera
Global cam:TCamera = CreateCamera(cpiv)
PositionEntity cam, -1.5 * 100.0, 0.5 * 100, 1.5 * 100.0
CameraRange cam, 0.1, ScreenX + 1

' fade quad
Global fade:TMesh = CreateQuad(10, Null, False, 1 + 2 + 16 + 32, 0, 0, 0, 1)
EntityParent fade, cam
PositionEntity fade, 0, 0, 0
ScaleEntity fade, ScreenX, ScreenY, 1
MoveEntity fade, 0, 0, ScreenX
EntityOrder fade, -100

' light
Global light:TLight = CreateLight(2)
PositionEntity light, 0, 0, -20
LightRange light, 20
AmbientLight 1, 1, 1

' earth
Global earth:TMesh = CreateSphere(64)
EntityPickMode earth, 1, 1
EntityRadius earth, 1.5
EntityTexture earth, masktex, 0, 0
EntityTexture earth, nighttex, 0, 1
EntityTexture earth, earthtex, 0, 2
TextureBlend masktex, 1
TextureBlend nighttex, 3
TextureBlend earthtex, 2

' cloudsphere
Global clouds:TMesh = CreateSphere(64, earth)
ScaleEntity clouds, 1.005, 1.005, 1.005
EntityTexture clouds, cloudtex, 0, 0
TextureBlend cloudtex, 5

' atmosphere shaders
Global atmo:Tmesh = CreatePlanetShader(16, 16, PR, PG, PB, earth)
Global glow1:Tmesh = CreateGlow(0.95, 1.05, 360, 1 + 2 + 32, 3, PR, PG, PB, 0.2, 0, 0, 0, 0)
Global glow2:Tmesh = InitGlow(earth, 0.5, 0.5)
ScaleMesh atmo, 1.0 / 8, 1.0 / 8, 1.0 / 8
EntityTexture atmo, atmotex
EntityBlend atmo, 3
EntityFX atmo, 2 + 16
ScaleTexture atmotex, -1, 1
EntityOrder glow1, 1
EntityOrder glow2, -2
EntityOrder atmo, -1

' moon
Global moon:Tmesh = CreateSphere(32, mpiv)
EntityPickMode moon, 1, 1
EntityRadius moon, 0.25
ScaleEntity moon, .277, .277, .277
MoveEntity moon, 3, 0, -3
EntityTexture moon, moontex

' sun
Global suntex:TTexture = CreateSunTexture()
Local size:Float = 1.0
Global SUN:TPivot = CreatePivot()
Global SUN0:TMesh = CreateQuad(size * 1.0, suntex, 3, 1 + 8, 255, 255, 255, 1.00)
Global SUN1:TMesh = CreateQuad(size * 1.5, suntex, 3, 1 + 8, 255, 192, 128, 1.00)
Global SUN2:TMesh = CreateQuad(size * 3.0, suntex, 3, 1 + 8, 255, 255, 224, 0.75)
Global SUN3:TMesh = CreateQuad(size * 15.0, suntex, 3, 1 + 8, 255, 255, 224, 0.50)
EntityParent SUN0, SUN
EntityParent SUN1, SUN
EntityParent SUN2, SUN
EntityParent SUN3, SUN
PositionEntity SUN, 0, 0, -20

' milkyway
Global milkyway:TPivot = InitMilkyway(500, 0.2, 10, 40, 500, 0, 0, 0, 0, 255, 192, 128, 0.5, 0, 0, 0, 0)

PlaySound backgroundmusic

MoveMouse ScreenX / 2, ScreenY / 2

HideMouse

Local s:Float, x:Float, y:Float, wait:Int, cl:Int, sh:Int, d:Float

' ----------------------------------------------------------------------------
' Main Loop
' ----------------------------------------------------------------------------
While Not AppTerminate()
	
	Cls
	
	' fade screen
	If fader > 0 Then
		fader = fader - 0.005
		UpdateMesh(fade, 0, 0, 0, fader)
	EndIf

	' SPACE = freeze
	If KeyHit(KEY_SPACE) Then wait = 1 - wait
	
	' C = show / hide clouds
	If KeyHit(KEY_C) Then
	
		cl = 1 - cl
		
		If cl = 1 Then
		
			HideEntity clouds
			
		Else
			ShowEntity clouds
			
		EndIf
		
	EndIf
	
	' S = show/hide "shaders"
	If KeyHit(KEY_S) Then
	
		sh = 1 - sh
		
		If sh = 1 Then
		
			HideEntity atmo
			HideEntity glow1
			HideEntity glow2
			
		Else
		
			ShowEntity atmo
			ShowEntity glow1
			ShowEntity glow2
			
		EndIf
		
	EndIf
	
	' Escape = Terminate
	If KeyHit(KEY_ESCAPE) Then End
	
	' only move if not waiting
	If wait = 0 Then
			
		s = s + 0.33 Mod 360.0
		x = x + 1 Mod 180.0
		y = y + 0.5 Mod 180.0
				
		' rotations
		PositionTexture masktex, -Normalize(s, 0, 360, 0, 1), 0
		RotateEntity earth, 0, s, 0
		TurnEntity clouds, 0, 0.02, 0
		TurnEntity cpiv, 0, .5, 0
		TurnEntity mpiv, 0, 0.63, 0
			
		' initial move
		d = 1 - (1.33 / Exp(EntityDistance(cam, earth) * 0.1))
				
		' moves
		MoveEntity cam, (Sin(y) / 150.0) + d, Cos(x * 0.25) / 180.0, (-Sin(y) / 150.0) + d
						
	EndIf
	
	' milkway holds position
	PositionEntity milkyway, EntityX(cam, 1), EntityY(cam, 1), EntityZ(cam, 1)
		
	' points
	PointEntity SUN, cam
	PointEntity atmo, cam
	PointEntity cam, earth
		
	' glows
	UpdateGlow(glow1, cam, earth, 1.0)
	UpdateGlow(atmo, cam, earth, 1.04)
		
	UpdateWorld
	RenderWorld
	
	' ENTER = save Screenshot
	If KeyHit(KEY_RETURN) Then Screenshot()
	
	Flip True

Wend

End

Function SetIcon(iconname$, TheWindow%)	
	?Win32
	Local icon:Int = ExtractIconA(TheWindow, iconname, 0)
	Local WM_SETICON:Int = $80
	Local ICON_BIG:Int = 1
	sendmessage(TheWindow, WM_SETICON, ICON_BIG, icon)
	?
End Function

Function Normalize:Float(value:Float = 128.0, value_min:Float = 0.0, value_max:Float = 255.0, norm_min:Float = 0.0, norm_max:Float = 1.0)
	
	Return ((value:Float - value_min:Float) / (value_max:Float - value_min:Float)) * (norm_max:Float - norm_min:Float) + norm_min:Float
		
End Function

Function CreateQuad:TMesh(scale:Float = 1.0, tex:TTexture = Null, blend:Int = 0, fx:Int = 0, r:Int = 255, g:Int = 255, b:Int = 255, a:Float = 1.0)
	
	Local mesh:TMesh = CreateMesh()
	Local surf:TSurface = CreateSurface(mesh)
	
	Local v0:Int = AddVertex(surf, 1, 1, 0, 0, 0)
	Local v1:Int = AddVertex(surf, -1, 1, 0, 1, 0)
	Local v2:Int = AddVertex(surf, -1, -1, 0, 1, 1)
	Local v3:Int = AddVertex(surf, 1, -1, 0, 0, 1)
	
	AddTriangle surf, v0, v1, v2
	AddTriangle surf, v0, v2, v3
	
	If fx Then EntityFX Mesh, fx
	If tex Then EntityTexture Mesh, tex
	If blend Then EntityBlend Mesh, blend
	
	EntityColor mesh, r, g, b
	EntityAlpha mesh, a
	
	VertexColor surf, v0, r, g, b, a
	VertexColor surf, v1, r, g, b, a
	VertexColor surf, v2, r, g, b, a
	VertexColor surf, v3, r, g, b, a
	
	ScaleEntity mesh, scale, scale, scale
		
	Return mesh
		
End Function

Function UpdateMesh(mesh:TMesh, r:Int = 255, g:Int = 255, b:Int = 255, a:Float = 1.0)
	
	Local surf:TSurface = GetSurface(mesh, 1)
		
	For Local v:Int = 0 To CountVertices(surf) - 1
	
		VertexColor surf, v, r, g, b, a
		
	Next
	
End Function

Function CreatePlanetShader:Tmesh(xs:Float = 10.0, ys:Float = 10.0, r:Int = 255, g:Int = 255, b:Int = 255, parent:Tmesh = Null)
	
	Local mesh:Tmesh = CreateMesh(parent)
	Local surf:Tsurface = CreateSurface(mesh)
	Local v0:Int, v1:Int, v2:Int, v3:Int
	
	Local x:Int, y:Int
		
	For x = 0 To xs - 1
			
		For y = 0 To ys - 1
									
			v0 = AddVertex(surf, x - xs / 2.0, -y + ys / 2.0, 0, x * 1.0 / xs, y * 1.0 / ys)
			v1 = AddVertex(surf, x - xs / 2.0, -y + ys / 2.0 - 1, 0, x * 1.0 / xs, (y + 1) * 1.0 / ys)
			v2 = AddVertex(surf, x - xs / 2.0 + 1, -y + ys / 2.0 - 1, 0, (x + 1) * 1.0 / xs, (y + 1) * 1.0 / ys)
			v3 = AddVertex(surf, x - xs / 2.0 + 1, -y + ys / 2.0, 0, (x + 1) * 1.0 / xs, y * 1.0 / ys)
						
		    	AddTriangle(surf, v3, v0, v1)
			AddTriangle(surf, v2, v3, v1)
						
			VertexColor surf, v0, r, g, b
			VertexColor surf, v1, r, g, b
			VertexColor surf, v2, r, g, b
			VertexColor surf, v3, r, g, b
						
			VertexNormal surf, v0, (x - xs / 2) / xs * 2, (-y + ys / 2.0) / ys * 2, 1
			VertexNormal surf, v1, (x - xs / 2) / xs * 2, (-y + ys / 2.0 - 1) / ys * 2, 1
			VertexNormal surf, v2, (x - xs / 2 + 1) / xs * 2, (-y + ys / 2.0 - 1) / ys * 2, 1
			VertexNormal surf, v3, (x - xs / 2 + 1) / xs * 2, (-y + ys / 2.0) / ys * 2, 1
						
		Next
		
	Next
	
	Return mesh
	
End Function

Function CreateGlow:Tmesh(radius1:Float = 1.0, radius2:Float = 2.0, segments:Int = 360, fx:Int = 0, blend:Int = 0, r1:Int = 255, g1:Int = 255, b1:Int = 255, al1:Float = 0.0, r2:Int = 0, g2:Int = 0, b2:Int = 0, al2:Float = 1.0)
	
	Local a1:Float, a2:Float, a3:Float, a4:Float, angle:Int, v0:Int, v1:Int, v2:Int, v3:Int
	Local mesh:Tmesh = CreateMesh()
	Local surf:TSurface = CreateSurface(mesh)
		
	If segments > 360 Then segments = 360
		
	For angle = 1 To segments
			
		a1 = angle * 360.0 / segments
		a2 = angle * 360.0 / segments + 360.0 / segments
		a3 = angle * 360.0 / segments + 180.0 / segments
		a4 = angle * 360.0 / segments - 180.0 / segments
				
		v0 = AddVertex(surf, radius1 * Cos(a1), radius1 * Sin(a1), 0, 0, 0)
		v1 = AddVertex(surf, radius1 * Cos(a2), radius1 * Sin(a2), 0, 0, 0)
		v2 = AddVertex(surf, radius2 * Cos(a3), radius2 * Sin(a3), 0, 1, 1)
		v3 = AddVertex(surf, radius2 * Cos(a4), radius2 * Sin(a4), 0, 0, 1)
				
		VertexColor surf, v0, r1, g1, b1, al1
		VertexColor surf, v1, r1, g1, b1, al1
		VertexColor surf, v2, r2, g2, b2, al2
		VertexColor surf, v3, r2, g2, b2, al2
			
		AddTriangle surf, v2, v1, v0
		AddTriangle surf, v0, v3, v2
			
	Next
	
	If fx > 0 Then EntityFX mesh, fx
	If blend > 0 Then EntityBlend mesh, blend
		
	Return mesh
	
End Function

Function InitGlow:Tmesh(parent:Tmesh = Null, shininess:Float, glowalpha:Float)
	
	Local mesh:TMesh = CreateSphere(64, parent)
	
	Local faketex:TTexture = CreateFakeShaderTexture()
			
	EntityBlend mesh, 3
	EntityTexture mesh, faketex, 0, 1
		
	EntityFX mesh, 2
	EntityShininess mesh, shininess
		
	UpdateMesh(mesh, 64, 96, 255, glowalpha)
		
	Return mesh
	
End Function

Function UpdateGlow(mesh:TMesh, cam:TCamera, parent:TMesh, scale:Float = 1.0)
	
	Local radius:Float, distance:Float
	Local c1:Float, a1:Float, q1:Float, p1:Float, h1:Float, alpha1:Float, beta1:Float, gamma1:Float, alpha2:Float, b2:Float, c2:Float
		
	PointEntity mesh, cam
		
	radius = 1
	distance = EntityDistance(cam, parent)
		
	c1 = distance
	a1 = radius
	q1 = a1 ^ 2 / c1
	p1 = c1 - q1
	h1 = Sqr(p1 * q1)
	gamma1 = 90
	alpha1 = ATan(h1 / p1)
	beta1 = gamma1 - alpha1
		
	alpha2 = 90 - (90 - beta1)
	b2 = a1 / Tan(alpha2)
	c2 = (Sqr(a1 ^ 2 + b2 ^ 2)) / radius * scale
		
	ScaleEntity mesh, c2, c2, c2
		
End Function

Function CreateSunTexture:TTexture()
	
	Local pixmap:TPixmap = CreatePixmap(512, 512, PF_RGBA8888)
	
	Local i:Float, j:Int, col:Int, rgb:Int
	
	For j = 0 To 255
		
		col=255-j
		If col > 255 Then col = 255
		rgb=col*$1000000+col*$10000+col*$100+col
		
		For i=0 To 360 Step 0.1
			
			WritePixel(pixmap, 256 + (Sin(i) * j), 256 + (Cos(i) * j), rgb)
			
		Next
		
	Next
	
	SavePixmapPNG(pixmap, "textures/sun.png")
	
	Local tex:TTexture = LoadTexture("textures/sun.png", 2 + 8)
	
	Return tex
	
End Function

Function CreateFakeShaderTexture:TTexture()
	
	Local pixmap:TPixmap = CreatePixmap(512, 512, PF_RGBA8888)
	
	Local x:Int, y:Int, i:Float, j:Int, col:Int, rgb:Int
		
	For x = 0 To 511
		
		For y=0 To 511
			
			rgb=255*$1000000+255*$10000+255*$100+255
			WritePixel(pixmap, x, y, rgb)
			
		Next
		
	Next
	
	For j = 0 To 255
		
		col=j*1.0/Exp((255-j)*0.02)
		
		If col>255 Then col=255
		If col<0 Then col=0
		
		rgb=col*$1000000+col*$10000+col*$100+col
		
		For i=0 To 360 Step 0.1
			
			WritePixel(pixmap, 256 + (Sin(i) * j), 256 + (Cos(i) * j), rgb)
			
		Next
		
	Next

	SavePixmapPNG(pixmap, "textures/fakeshader.png")
	
	Local tex:TTexture = LoadTexture("textures/fakeshader.png", 64)
	
	Return tex
	
End Function

Function Screenshot()

	Local img:TPixmap = GrabPixmap(0, 0, ScreenX, ScreenY)
	SavePixmapPNG(img, MilliSecs() + ".png")

EndFunction