Const StarTexScale:Float = 1.5
Const PanoSegments:Int = 32

Global startex:TTexture = LoadTexture("textures/star.png", 2 + 8)
Global starstex:TTexture = LoadTexture("textures/stars.png", 2 + 8)
Global fogtex:TTexture = LoadTexture("textures/milkyway.jpg", 8 + 16 + 32)

Function InitMilkyway:TPivot(Stars:Int, Dist:Float, MinAngle:Float, MaxAngle:Float, Scale:Float, R1:Int = 0, G1:Int = 0, B1:Int = 0, A1:Float = 0.0, R2:Int = 0, G2:Int = 0, B2:Int = 0, A2:Float = 0.0, R3:Int = 0, G3:Int = 0, B3:Int = 0, A3:Float = 0.0)
	
	Local MilkyWay:TPivot = CreatePivot()
	
	' create stars
	Local Starbox:TMesh = InitStarBox(Scale, StarsTex, 1, 0)
	Local FogPanorama:TMesh = InitPanorama(FogTex, Scale, Scale * 0.8, PanoSegments, 1 + 2 + 32, R1, G1, B1, A1, R2, G2, B2, A2, R3, G3, B3, A3)
	Local StarSphere:Tmesh = InitStarSphere(StarTex, Stars, 0.5 * (Scale / 100.0), 0.75 * (Scale / 50.0), 0.5, Dist, MinAngle, MaxAngle, Scale, 1)
				
	EntityParent Starbox, MilkyWay
	EntityParent FogPanorama, MilkyWay
	EntityParent StarSphere, MilkyWay
	
	' reorder
	EntityOrder StarSphere, 1
	EntityOrder FogPanorama, 2
	EntityOrder Starbox, 3
	
	' blending
	TextureBlend fogtex, 5
	EntityBlend FogPanorama, 3
	EntityBlend Starbox, 3
	EntityBlend StarSphere, 3
	
	' scaling
	ScaleTexture starstex, StarTexScale, StarTexScale
		
	Return MilkyWay
	
End Function

Function InitStarBox:TMesh(scale:Float, texture:TTexture = Null, fx:Int, blend:Int)
	
	Local starbox:Tmesh = CreateCube()
	EntityTexture starbox, texture
	FlipMesh starbox
	ScaleEntity starbox, scale, scale, scale
	
	EntityFX starbox, fx
	EntityBlend starbox, blend
		
	Return starbox
	
End Function

Function InitPanorama:TMesh(texture:TTexture = Null, radius:Float, h:Float, segmente:Int, fx:Int = 0, r1:Int = 255, g1:Int = 255, b1:Int = 255, a1:Float = 1.0, r2:Int = 255, g2:Int = 255, b2:Int = 255, a2:Float = 1.0, r3:Int = 255, g3:Int = 255, b3:Int = 255, a3:Float = 1.0)
	
	Local ang1:Float, ang2:Float, inc:Float, x1:Float, z1:Float, x2:Float, z2:Float
	Local uu1:Float, uu2:Float, uv1:Float, uv2:Float, uv3:Float
	Local v0:Int, v1:Int, v2:Int, v3:Int, v4:Int, v5:Int
		
	Local mesh:TMesh = CreateMesh()
	Local surf:TSurface = CreateSurface(mesh)
		
	inc = 360.0 / segmente
		
	ang1 = 0
		
	While ang1 < 360
			
		ang2 = (ang1 + inc) Mod 360
				
		x1 = radius * Cos(ang1)
		z1 = radius * Sin(ang1)
		x2 = radius * Cos(ang2)
		z2 = radius * Sin(ang2)
				
		If ang1 < 180 Then
					
			' use upper texture UV coordinates
			uu1 = 1.0 - (ang1 / 180.0)
			uu2 = 1.0 - (ang1 + inc) / 180.0
			uv1 = 0.50
			uv2 = 0.25
			uv3 = 0.00
						
		Else
			
			' use lower texture UV coordinates
			uu1 = 1 - (((ang1 / 180.0)) - 1)
			uu2 = 1 - (((ang1 + inc) / 180.0) - 1)
			uv1 = 1.0
			uv2 = 0.75
			uv3 = 0.5
						
		EndIf
		
		v0 = AddVertex(surf, x1, -h, z1, uu1, uv1)
		v1 = AddVertex(surf, x2, -h, z2, uu2, uv1)
		v2 = AddVertex(surf, x1, 0, z1, uu1, uv2)
		v3 = AddVertex(surf, x2, 0, z2, uu2, uv2)
		v4 = AddVertex(surf, x1, h, z1, uu1, uv3)
		v5 = AddVertex(surf, x2, h, z2, uu2, uv3)
		
		VertexColor surf, v0, r1, g1, b1, a1
		VertexColor surf, v1, r1, g1, b1, a1
		VertexColor surf, v2, r2, g2, b2, a2
		VertexColor surf, v3, r2, g2, b2, a2
		VertexColor surf, v4, r3, g3, b3, a3
		VertexColor surf, v5, r3, g3, b3, a3
		
		AddTriangle surf, v0, v1, v3
		AddTriangle surf, v3, v2, v0
		AddTriangle surf, v2, v5, v4
		AddTriangle surf, v5, v2, v3
		
		ang1 = ang1 + inc
				
	Wend
	
	EntityFX mesh, fx
		
	EntityTexture mesh, texture
		
	Return mesh
	
End Function

Function InitStarSphere:TMesh(texture:TTexture = Null, stars:Int, mini:Float, maxi:Float, fix:Float, centered:Float, centermin:Float, centermax:Float, range:Float, scale:Float = 1.0)
	
	Local i:Int, size:Float, a:Float
	Local r:Int, g:Int, b:Int
	Local col:Float, maxy:Float
		
	Local star:TMesh = CreateStarQuad()
		
	Local mesh:TMesh = CreateMesh()
	Local surf:TSurface = CreateSurface(mesh)
	EntityFX mesh, 1 + 2
	
	For i=1 To stars
		
		' create new surface if there are too many stars
		If CountVertices(surf) >= 32000 Then surf = CreateSurface(mesh)
				
		' size
		size = mini * scale
		If Rnd(1) > fix Then size = Rnd(mini, maxi) * scale
		ScaleEntity star, size, size, size
				
		' reset quad
		PositionEntity star, 0, 0, 0
				
		' more stars centered or random?
		If Rnd(1) > centered Then maxy = centermin Else maxy = centermax
				
		' turn and move starquad
		TurnEntity star, Rnd(-Rnd(0, maxy), Rnd(0, maxy)), Rnd(0, Rnd(0, 360)), 0
		MoveEntity star, 0, 0, range * scale
		PointEntity star, mesh
				
		' color
		col = Rnd(1)
		
		' 25% orange stars
		If col > 0.75 And col <= 1.00 Then
			r = 255
			g = 224
			b = 192
		EndIf
		
		' 25% yellow stars
		If col > 0.50 And col <= 0.75 Then
			r = 255
			g = 255
			b = 192
		EndIf
		
		' 25% blue stars	
		If col > 0.25 And col <= 0.50 Then
			r = 192
			g = 224
			b = 255
		EndIf
		
		' 25% white stars
		If col > 0.00 And col <= 0.25 Then
			r = 255
			g = 255
			b = 255
		EndIf
		
		' alpha
		If size > mini * scale Then
			a = 1.0
		Else
			a = Rnd(0.5, 1)
		EndIf
				
		' add to single surface mesh
		AddToSurface(star, surf, mesh, r, g, b, a)
				
	Next
	
	FreeEntity star
		
	EntityTexture mesh, texture
		
	FlipMesh mesh
		
	Return mesh
	
End Function

Function CreateStarQuad:TMesh(r:Int = 255, g:Int = 255, b:Int = 255, a:Float = 1.0, fx:Int = 0)
	
	Local v1:Int, v2:Int, v3:Int, v4:Int
		
	Local mesh:TMesh = CreateMesh()
	Local surf:TSurface = CreateSurface(mesh)
		
	v1 = AddVertex(surf, -1, 1, 0, 1, 0)
	v2 = AddVertex(surf, 1, 1, 0, 0, 0)
	v3 = AddVertex(surf, -1, -1, 0, 1, 1)
	v4 = AddVertex(surf, 1, -1, 0, 0, 1)
	
	VertexColor surf, v1, r, g, b, a
	VertexColor surf, v3, r, g, b, a
	VertexColor surf, v2, r, g, b, a
	VertexColor surf, v4, r, g, b, a
	
	AddTriangle(surf, 0, 1, 2)
	AddTriangle(surf, 3, 2, 1)
		
	EntityFX mesh, fx
		
	Return mesh
	
End Function

Function AddToSurface(mesh:TMesh, surf:TSurface, singlesurfaceentity:TMesh, r:Int, g:Int, b:Int, a:Float)
	
	Local v:Int[3]
		
	Local surface:TSurface = GetSurface(mesh, 1)
		
	For Local i1:Int = 0 To CountTriangles(surface) - 1
			
		For Local i2:Int = 0 To 2
					
			Local oldv:Int = TriangleVertex(surface, i1, i2)
			TFormPoint VertexX(surface, oldv), VertexY(surface, oldv), VertexZ(surface, oldv), mesh, singlesurfaceentity
			v[i2] = AddVertex(surf, TFormedX(), TFormedY(), TFormedZ(), VertexU(surface, oldv), VertexV(surface, oldv))
			VertexColor surf, v[i2], r, g, b, a
						
		Next 
		
		AddTriangle(surf, v[0], v[1], v[2])
				
	Next 
	
End Function